<?php

/**
 * Tencent Cloud SMS
 * @author rehiy
 * @url https://www.rehiy.com/post/527/
 */

class QCloudBasic
{
    protected $domain = 'tencentcloudapi.com';
    protected $service = '';
    protected $version = '';
    protected $secretId = '';
    protected $secretKey = '';

    protected function post($region, $action, $data)
    {
        $timestamp = time();
        $payload = json_encode($data, 320);
        $authorization = $this->signature($action, $payload, $timestamp);

        $headers = [
            "Authorization: {$authorization}",
            "Content-Type: application/json; charset=utf-8",
            "Host: {$this->domain}",
            "X-TC-Action: {$action}",
            "X-TC-Timestamp: {$timestamp}",
            "X-TC-Version: {$this->version}",
            "X-TC-Region: {$region}"
        ];

        return $this->httpRequest('POST', "https://{$this->domain}", $payload, $headers);
    }

    protected function signature($action, $payload = '{}', $timestamp = 0)
    {
        $algorithm = 'TC3-HMAC-SHA256';
        $date = gmdate('Y-m-d', $timestamp);

        $canonicalUri = '/';
        $canonicalQuery = '';
        $canonicalHeaders = implode("\n", [
            'content-type:application/json; charset=utf-8',
            'host:' . $this->domain,
            'x-tc-action:' . strtolower($action),
            ''
        ]);

        $signedHeaders = 'content-type;host;x-tc-action';
        $canonicalRequest = implode("\n", [
            'POST',
            $canonicalUri,
            $canonicalQuery,
            $canonicalHeaders,
            $signedHeaders,
            hash('SHA256', $payload)
        ]);

        $credentialScope = "{$date}/{$this->service}/tc3_request";
        $stringToSign = implode("\n", [
            $algorithm,
            $timestamp,
            $credentialScope,
            hash('SHA256', $canonicalRequest),
        ]);

        $secretDate = hash_hmac('SHA256', $date, 'TC3' . $this->secretKey, true);
        $secretService = hash_hmac('SHA256', $this->service, $secretDate, true);
        $secretSigning = hash_hmac('SHA256', 'tc3_request', $secretService, true);
        $signature = hash_hmac('SHA256', $stringToSign, $secretSigning);

        return implode(', ', [
            "{$algorithm} Credential={$this->secretId}/{$credentialScope}",
            "SignedHeaders={$signedHeaders}",
            "Signature={$signature}"
        ]);
    }

    protected function httpRequest($method, $url, $data = null, $header = null)
    {
        $ch = curl_init($url);

        curl_setopt($ch, CURLOPT_TIMEOUT, 25);
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header ?? []);

        if ($data && $method == 'POST') {
            if (is_array($data)) {
                if (in_array('Content-Type: application/json', $header)) {
                    $data = json_encode($data, 320);
                } else {
                    $data = http_build_query($data);
                }
            }
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        }

        list($body, $errno, $error) = [
            curl_exec($ch), curl_errno($ch), curl_error($ch), curl_close($ch)
        ];

        if ($errno != 0) {
            throw new \Exception('CURL - ' . $error, $errno);
        }

        try {
            $res = json_decode($body, true);
            return $res['Response'] ?? '';
        } catch (\Exception $e) {
            return false;
        }
    }
}

class QCloudSms extends QCloudBasic
{
    protected $service = 'sms';
    protected $version = '2021-01-11';
    protected $sdkAppId = '';
    protected $signName = '';
    protected $templateCode = '';

    public function __construct($config)
    {
        $this->domain = $this->service . '.' . $this->domain;
        foreach ($config as $key => $value) {
            $this->$key = $value;
        }
    }

    public function send($phoneNumbers, $templateParam, $extendCode = '', $sessionContext = '')
    {
        $data = [
            'PhoneNumberSet' => $phoneNumbers,
            'SmsSdkAppId' => $this->sdkAppId,
            'SignName' => $this->signName,
            'TemplateId' => $this->templateCode,
            'TemplateParamSet' => $templateParam,
        ];
        if ($extendCode) {
            $data['ExtendCode'] = $extendCode;
        }
        if ($sessionContext) {
            $data['SessionContext'] = $sessionContext;
        }
        return $this->post('ap-guangzhou', 'SendSms', $data);
    }
}

function send_sms($phone, $code)
{
    $c_china = [
        'secretId' => 'AK----------------------',
        'secretKey' => 'SK----------------------',
        'sdkAppId' => '1400000000',
        'signName' =>  '签名',
        'templateCode' => '123456',
    ];

    $c_global = [
        'secretId' => 'AK----------------------',
        'secretKey' => 'SK----------------------',
        'sdkAppId' => '1400000000',
        'signName' =>  '签名',
        'templateCode' => '654321',
    ];

    $phone = preg_replace('/^\+86(\d{11})$/', '$1', $phone);
    $config = strpos($phone, '+') === 0 ? $c_global : $c_china;

    return (new QCloudSms($config))->send([$phone], [$code]);
}

print_r(send_sms('+8613800000000', '123456'));
print_r(send_sms('+188800000000', '123456'));
